#include "stdafx.h"
#include "7134api.h"

#define FM_TUNE             1
#define FM_TUNEMUTED        2
#define FM_MUTEUNMUTE       4
#define FM_GETVOLUME        8
#define FM_SETVOLUMEBYVALUE 16
#define FM_SETVOLUMEUPDOWN  32
#define FM_BASSTREBLE       64
#define FM_ISSTEREO         128
#define FM_SETSTEREO        256
#define FM_GETSIGNAL        512
#define FM_AMRANGE          1024
#define FM_RDS              2048
#define FM_SCANSTATION      4096
#define FM_CONFIGURATIONDIALOG 65536

ICaptureGraphBuilder * pGraphBuilder = NULL;
IFilterGraph * pFilterGraph = NULL;
IBaseFilter * pRadio = NULL;
IBaseFilter * pOverlayMixer = NULL;
IAMCrossbar * pAMCrossbar = NULL;
IAMTVTuner * pAMTVTuner = NULL;

char * errmsg;

bool XBar_Setup(void)
{
    long InPinCount, OutPinCount;
	long RelatedPin, PhysicalType;
	long Pin_AudioTuner, Pin_AudioDecoder = -1;
	long Pin_VideoTuner, Pin_VideoDecoder = -1;
	
	HRESULT hr;

	hr = pAMCrossbar->get_PinCounts(&OutPinCount, &InPinCount);
    if (hr != S_OK) {
        errmsg = "Error during crossbar setup";
        return false;
    }

	for (int i=0; i<InPinCount; i++) {
	  hr = pAMCrossbar->get_CrossbarPinInfo(true, i, &RelatedPin, &PhysicalType);
	  if ((hr == S_OK) && (PhysicalType == PhysConn_Audio_Tuner)) Pin_AudioTuner = i;
	  if ((hr == S_OK) && (PhysicalType == PhysConn_Video_Tuner)) Pin_VideoTuner = i;
    }

	for (i=0; i<OutPinCount; i++) {
	  hr = pAMCrossbar->get_CrossbarPinInfo(false, i, &RelatedPin, &PhysicalType);
	  if ((hr == S_OK) && (PhysicalType == PhysConn_Audio_AudioDecoder)) Pin_AudioDecoder = i;
	  if ((hr == S_OK) && (PhysicalType == PhysConn_Video_VideoDecoder)) Pin_VideoDecoder = i;
    }

    if (Pin_AudioTuner == -1)
	{
		errmsg = "Audio tuner not found";
		return false;
	}

    if (Pin_AudioDecoder == -1)
	{
		errmsg = "Audio decoder not found";
		return false;
	}

	if (pAMCrossbar->CanRoute(Pin_AudioDecoder, Pin_AudioTuner) != S_OK)
	{
		errmsg = "Cannot route audio";
		return false;
	}

    if (pAMCrossbar->Route(Pin_AudioDecoder, Pin_AudioTuner) != S_OK)
	{
		errmsg = "Cannot route audio";
		return false;
	}

	pAMCrossbar->Route(Pin_VideoDecoder, Pin_VideoTuner);

    return true;
}

bool Tuner_Setup()
{

    HRESULT hr;

    hr = pGraphBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, pRadio, IID_IAMTVTuner, (void **) &pAMTVTuner);
    if (hr != S_OK) {
        errmsg = "Tuner_Setup: Cannot find radio tuner";
        return false;
    }

	hr = pAMTVTuner->put_InputType(0, TunerInputAntenna);
    if (hr != S_OK) {
        errmsg = "Tuner_Setup: Cannot set input type";
        return false;
    }

	hr = pAMTVTuner->put_Mode(AMTUNER_MODE_FM_RADIO);
    if (hr != S_OK) {
        errmsg = "Tuner_setup: Cannot set FM mode";
        return false;
    }

    return true;
}

bool Play()
{
    HRESULT hr;
	IMediaControl * pMediaControl;

    if (!pFilterGraph) {
        errmsg = "Filter graph not present";
        return false;
    }

	hr = pFilterGraph->QueryInterface(IID_IMediaControl, (void **) &pMediaControl);
    if (hr != S_OK) {
        errmsg = "Cannot get media control interface";
        return false;
    }

	hr = pMediaControl->Run();
    pMediaControl->Release();
    if ((hr != S_OK) && (hr != S_FALSE)) {
        char buffer[200];
        sprintf(buffer,"Error playing stream (%X)",hr);
        errmsg = strdup(buffer);
        return false;
    }

    return true;
}

bool GetCaptureDevice(void)
{
    ICreateDevEnum * pDeviceEnum;
    IEnumMoniker * pEnumMoniker;
    HRESULT hr;

	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER, IID_ICreateDevEnum, (void **) &pDeviceEnum);
    if (hr != S_OK) {
        errmsg = "Cannot instatiate device enumerator";
        return false;
    }

	hr = pDeviceEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pEnumMoniker, NULL);
	pDeviceEnum->Release();
    if (hr != S_OK) {
        errmsg = "Error enumerating capture devices";
        return false;
    }

    IMoniker * pMoniker;
    pEnumMoniker->Reset();
	hr = pEnumMoniker->Next(1, &pMoniker, NULL);
	pEnumMoniker->Release();
    if (hr != S_OK) {
        errmsg = "Error enumerating capture devices";
        return false;
    }

    hr = pMoniker->BindToObject(NULL, NULL, IID_IBaseFilter, (void **) &pRadio);
	pMoniker->Release();
    if (hr != S_OK) {
        errmsg = "Capture device not found";
        return false;
    }

    return true;
}

bool WDM_Init(void)
{
    CoInitialize(NULL);

    HRESULT hr;

    hr = CoCreateInstance(CLSID_CaptureGraphBuilder, NULL, CLSCTX_INPROC, IID_ICaptureGraphBuilder, (void **) &pGraphBuilder);
    if (hr != S_OK) {
        errmsg = "Cannot instantiate graph builder";
        return false;
    }

    hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC, IID_IFilterGraph, (void **) &pFilterGraph);
    if (hr != S_OK) {
        errmsg = "Cannot instantiate filtergraph";
        return false;
    }
	
    hr = pGraphBuilder->SetFiltergraph((IGraphBuilder *) pFilterGraph);
    if (hr != S_OK) {
        errmsg = "Cannot give graph to builder";
        return false;
    }

    if (!GetCaptureDevice()) return false;

	hr = pFilterGraph->AddFilter(pRadio, NULL);
    if (hr != S_OK) {
        errmsg = "Cannot add vidcap to filtergraph";
        return false;
    }

	// Rendrujem cez OverlayMixer aby mi nenabehlo okno ActiveMovie.
	// Pravdepodobne sa to da riesit aj elegantnejsie

    hr = CoCreateInstance(CLSID_OverlayMixer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **) &pOverlayMixer);
    if (hr != S_OK) {
        errmsg = "Cannot instantiate overlay mixer";
        return false;
    }

	hr = pFilterGraph->AddFilter(pOverlayMixer, NULL);
    if (hr != S_OK) {
        errmsg = "Cannot add overlay mixer to graph";
        return false;
    }

    IAMAnalogVideoDecoder * pAMAnalogVideoDecoder;
	hr = pGraphBuilder->FindInterface(&PIN_CATEGORY_PREVIEW, pRadio, IID_IAMAnalogVideoDecoder, (void **) &pAMAnalogVideoDecoder);
    if (hr != S_OK) {
        errmsg = "Cannot find analog decoder";
        return false;
    }
	pAMAnalogVideoDecoder->put_TVFormat(AnalogVideo_PAL_B);
	pAMAnalogVideoDecoder->Release();

	hr = pGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE, pRadio, NULL, pOverlayMixer );
    if (hr != S_OK) {
        char buffer[200];
        sprintf(buffer,"Error rendering stream (%X)",hr);
        errmsg = strdup(buffer);
        return false;
    }

	hr = pGraphBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, pRadio, IID_IAMCrossbar, (void **) &pAMCrossbar);
    if (hr != S_OK) {
        errmsg = "Cannot find crossbar device";
        return false;
    }

    if (!XBar_Setup()) return false;
    if (!Tuner_Setup()) return false;
    if (!Play()) return false;

    return true;
}

bool WDM_DeInit()
{
    if (pRadio) {
        pRadio->Stop();
        pRadio->Release();
        pRadio = NULL;
    }

    if (pOverlayMixer) {
        pOverlayMixer->Stop();
        pOverlayMixer->Release();
        pOverlayMixer = NULL;
    }

    if (pAMCrossbar) {
        pAMCrossbar->Release();
        pAMCrossbar = NULL;
    }

    if (pAMTVTuner) {
        pAMTVTuner->Release();
        pAMTVTuner = NULL;
    }


    if (pFilterGraph) {
        pFilterGraph->Release();
        pFilterGraph = NULL;
    }

    if (pGraphBuilder) {
        pGraphBuilder->Release();
        pGraphBuilder = NULL;
    }

    CoUninitialize();
    return true;
}

void _stdcall SetMute(bool bMute)
{
    SetMute713x(!bMute);
}

unsigned short _stdcall GetSignal(void)
{
    return (GetSignalStrength() & 0x40) ? 0 : 0xffff;
}

char * _stdcall GetModuleName(void)
{
    return "ManliTV FM Radio (SAA713x)";
}

long _stdcall GetModuleInfo(void)
{
    return FM_TUNE + FM_GETSIGNAL + FM_MUTEUNMUTE;
}

void _stdcall TuneFreq(long Freq)
{
    if (pAMTVTuner) pAMTVTuner->put_Channel(Freq * 1000,0,0);
    
}

void _stdcall TuneFreqMuted(long Freq)
{
    SetMute(true);
    if (pAMTVTuner) pAMTVTuner->put_Channel(Freq * 1000,0,0);
}

bool _stdcall HWInit(void)
{
    HCURSOR hCursor;

    hCursor = LoadCursor(NULL,IDC_WAIT);
    SetCursor(hCursor);
    BOOL success = InitAPI();
    if (success) success = WDM_Init();
    SetCursor(NULL);
    DestroyCursor(hCursor);

    if (success) 
        return true;
    else 
    {
        WDM_DeInit();
        MessageBox(0,errmsg,"Error initializing radio card",MB_OK + MB_ICONERROR);
        return false;
    }
}

bool _stdcall HWDeInit(void)
{
    HCURSOR hCursor;

    hCursor = LoadCursor(NULL,IDC_WAIT);
    SetCursor(hCursor);
    WDM_DeInit();
    SetCursor(NULL);
    DestroyCursor(hCursor);
    return true;
}

void _stdcall ConfigurationDialog(void)
{
    MessageBox(0,"haha","huhu",MB_OK);
}

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
